import { is } from './is.js'

/**
 * A replacement for the `in` operator (not to be confused with the `for-in` loop) that works properly.
 */
export const has = (/** @type {any} */ key, /** @type {any} */ ref) =>
  (is(String, key) || is(Number, key) || is(Symbol, key)) && Object.hasOwnProperty.call(ref ?? Object, key)

export const tests = {
  has: () => {
    const obj = { key: 'K', null: 'N' }

    console.assert('key' in obj)
    console.assert(has('key', obj))

    console.assert('null' in obj)
    console.assert(has('null', obj))

    // @ts-ignore
    console.assert(null in obj)
    console.assert(!has(null, obj))

    console.assert('toString' in obj)
    console.assert(!has('toString', obj))
  },

  'has: null': () => {
    let typeError

    try {
      // @ts-ignore
      console.assert('key' in null)
    } catch (error) {
      typeError = error
    }

    console.assert(typeError instanceof TypeError) // Cannot use 'in' operator to search for 'key' in null
    console.assert(!has('key', null))
  }
}
