/**
 * The type of arguments of the `jcss` helper.
 */
export type JcssNode = {
    [attributeOrSelector: string]: string | number | JcssNode | undefined;
};

/**
 * The type of arguments of the `jcss` helper.
 */
export type JcssRoot = Partial<Record<string, JcssNode>>;

/**
 * A simple CSS-in-JS helper.
 *
 * The `root` parameter provides a hierarchical description of CSS rules.
 *
 * - Keys of sub-objects whose values are NOT objects are treated as CSS attribute, and values are treated as values of those CSS attributes; the concatenation of keys of all parent objects is a CSS rule.
 * - All keys ignore the part starting with a splitter (default: `$$`) sign until the end of the key (e.g. `src$$1` → `src`, `@font-face$$1` → `@font-face`).
 * - In keys specifying CSS attribute, all uppercase letters are replaced by lowercase letters with an additional `-` character preceding them (e.g. `fontFamily` → `font-family`).
 * - Commas in keys that makes a CSS rule cause it to “split” and create separate rules for each part (e.g. `{div:{margin:1,'.a,.b,.c':{margin:2}}}` → `div{margin:1}div.a,div.b,div.c{margin:2}`).
 * - Top-level keys that begin with `@` are not concatenated with sub-object keys.
 */
export function jcss(root: JcssRoot, splitter?: string): string;

export const tests: {
    'jcss: #1': () => void;
    'jcss: #2': () => void;
    'jcss: #3': () => void;
    'jcss: #4': () => void;
    'jcss: #5': () => void;
    'jcss: #6': () => void;
};
