import { is } from './is.js'

/**
 * Language translations helper.
 */
export const locale = (
  /** @type {Partial<Record<string, Partial<Record<string, string>>>>} */ map,
  /** @type {string} */ defaultVersion
) => (
  /** @type {string} */ text,
  /** @type {string} */ version = defaultVersion
) => {
  const textV = map?.[version]?.[text]
  const textD = map?.[defaultVersion]?.[text]

  return is(String, textV) ? textV : is(String, textD) ? textD : text
}

export const tests = {
  locale: () => {
    const _ = locale({
      default: { Password: 'Hasło' },
      button: { Login: 'Zaloguj' }
    }, 'default')

    console.assert(_('Login') === 'Login')
    console.assert(_('Password') === 'Hasło')

    console.assert(_('Undefined text') === 'Undefined text')

    console.assert(_('Login', 'button') === 'Zaloguj')

    console.assert(_('Password', 'undefined_version') === 'Hasło')
    console.assert(_('Undefined text', 'undefined_version') === 'Undefined text')

    console.assert(_('toString') === 'toString')
    console.assert(_('toString', 'undefined_version') === 'toString')
  }
}
