import { eq } from './eq.js'

/**
 * A helper that protects calls to nested properties by a `Proxy` that initializes non-existent values with an empty object.
 */
// @ts-ignore
export const pro = (/** @type {any} */ ref) => new Proxy(ref, {
  get (target, key) {
    return pro(target[key] = target[key] ?? {})
  }
})

export const tests = {
  pro: () => {
    const ref = {}

    pro(ref).one.two[3][4] = 1234

    console.assert(eq(ref, { one: { two: { 3: { 4: 1234 } } } }))

    pro(ref).one.two.tree = 123

    console.assert(eq(ref, { one: { two: { 3: { 4: 1234 }, tree: 123 } } }))

    pro(ref).one.two = undefined

    console.assert(eq(ref, { one: { two: undefined } }))

    delete pro(ref).one.two

    console.assert(eq(ref, { one: {} }))

    pro(ref).one.two.three.four

    console.assert(eq(ref, { one: { two: { three: { four: {} } } } }))

    pro(ref).one.two.three.four = 1234

    console.assert(eq(ref, { one: { two: { three: { four: 1234 } } } }))
  }
}
