const ZEROS = '0'.repeat(16)
let counter = 0

/**
 * A helper that generates a UUID v1 identifier (with a creation timestamp).
 *
 * - The optional `node` parameter should have the format `/^[0123456789abcdef]+$/`.
 *   Its value will be trimmed to last 12 characters and left padded with zeros.
 */
export const uuid1 = ({
  date = new Date(),
  node = Math.random().toString(16).slice(2)
} = {}) => {
  const time = ZEROS + (10000 * (+date + 12219292800000)).toString(16)

  counter = (counter + 1) & 16383

  return time.slice(-8).concat(
    '-',
    time.slice(-12, -8),
    // @ts-ignore
    -1,
    time.slice(-15, -12),
    '-',
    (8 | (counter >> 12)).toString(16),
    (ZEROS + (counter & 4095).toString(16)).slice(-3),
    '-',
    (ZEROS + node).slice(-12))
}

export const tests = {
  uuid1: () => {
    for (let i = 1; i <= 22136; ++i) {
      const uuid = uuid1()

      i === 1 && console.assert(uuid.split('-')[3] === '8001')
      i === 4095 && console.assert(uuid.split('-')[3] === '8fff')
      i === 4096 && console.assert(uuid.split('-')[3] === '9000')
      i === 9029 && console.assert(uuid.split('-')[3] === 'a345')
      i === 13398 && console.assert(uuid.split('-')[3] === 'b456')
      i === 16384 && console.assert(uuid.split('-')[3] === '8000')
      i === 17767 && console.assert(uuid.split('-')[3] === '8567')
    }
  },

  'uuid1: node': () => {
    console.assert(uuid1({ node: '000123456789abc' }).split('-')[4] === '123456789abc')
    console.assert(uuid1({ node: '123456789' }).split('-')[4] === '000123456789')
  },

  'uuid1: date': () => {
    console.assert(uuid1({ date: new Date(323325000000) }).startsWith('c1399400-9a71-11bd'))
  }
}
